package no.edu.esign;

import no.edu.esign.model.sign.*;
import no.edu.esign.util.FileUtil;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

import java.io.File;
import java.util.List;
import java.util.Scanner;

import static no.edu.esign.model.sign.SignatureType.ADVANCED_SIGNATURE;
import static no.edu.esign.model.sign.SignatureType.AUTHENTICATED_SIGNATURE;

/**
 * Manual runner class to test getDocument, since the signing is manual process it is not automated
 * Required to set following args when running:
 * --ca.file.path="<path-to-pkcs12-formatted-file"
 * --ca.file.password="password for ca file"
 */
@SpringBootApplication
public class DocumentManualRunner implements CommandLineRunner {
    private static String NIN = "22899998107";
    private static final String BANKID_NIN = "10103933108";
    Logger logger = LoggerFactory.getLogger(DocumentManualRunner.class);

    @Autowired
    private SignerService signerService;

    public static void main(String[] args) {
        SpringApplication.run(DocumentManualRunner.class, args);
    }

    @Override
    public void run(String... args) {
        Scanner scanner = new Scanner(System.in);

        logger.info("Choose signatureType, {}/{}, def: {}", ADVANCED_SIGNATURE.name(), AUTHENTICATED_SIGNATURE.name(), ADVANCED_SIGNATURE.name());
        String inSignatureType = scanner.nextLine();
        logger.info("Choose securityLevel, 3/4, def: 4");
        int inSecurityLevel = scanner.nextInt();
        SignResponse signature = signerService.createSignature(createSignRequest(inSecurityLevel, inSignatureType));
        logger.info("Got signature response: "+  signature.toString());
        SignerDetail detail = signature.getSignersDetails().get(0);
        String signerRedirectUrl = detail.getRedirectUrl();
        logger.info("Get a new redirect url?(Enter 0 or 1) 0:no , 1:yes");
        int testRedirect = scanner.nextInt();
        if(testRedirect==1) {
            RedirectUrlResponse redirectUrlResponse = signerService.createNewRedirect(createRedirectUrlRequest(detail));
            String newRedirectUrl = redirectUrlResponse.getUpdatedRedirectUrl();
            logger.info("Old redirecturl was:" +signerRedirectUrl);
            logger.info("New redirecturl is:" + newRedirectUrl);
            signerRedirectUrl = newRedirectUrl;
        }
        logger.info("Opening browser...");
        BrowserUtil.open(signerRedirectUrl);
        logger.info("Perform signing with signers nin: "+  NIN);
        logger.info("Enter statusQueryToken: ");
        String statusQueryToken = scanner.nextLine();
        while (StringUtils.isBlank(statusQueryToken)){
            logger.info("Invalid statusQueryToken, try again");
            statusQueryToken = scanner.nextLine();
        }
        String statusQueryUrl = signature.getStatusQueryUrl();
        StatusRequest statusRequest = StatusRequest.builder()
                .statusQueryToken(statusQueryToken)
                .statusQueryUrl(statusQueryUrl)
                .build();

        StatusResponse statusResponse = signerService.getStatus(statusRequest);
        String msg = "Confimation url: %s, Pades reference: %s";
        logger.info(String.format(msg, statusResponse.getConfirmationUrl(), statusResponse.getPadesUrl()));

        DocumentRequest documentRequest = DocumentRequest.builder()
                .padesUrl(statusResponse.getPadesUrl())
                .build();
        DocumentResponse documentResponse = signerService.getDocuments(documentRequest);
        File singedDoc = FileUtil.base64ToPdf(documentResponse.getDocument(), "../esign/build/tmp/signedDocument.pdf");
        BrowserUtil.open(singedDoc.getAbsolutePath());

        System.exit(0);
    }

    private SignRequest createSignRequest(int inSecurityLevel, String inSignatureType) {
        // Given
        ExitUrls exitUrls = new ExitUrls();
        exitUrls.setCompletion("https://ntnu.no?");
        exitUrls.setRejection("https://ntnu.no?");
        exitUrls.setError("https://ntnu.no?");
        String description = "Signerings tittel";
        String title = "Dokument tittel";
        String fileName = "acrobat_digitalSignatures.pdf";
        String pdfEncoded = Base64.encodeBase64String(FileUtil.getPdfDocAsBytes(fileName));
        SignatureType signatureType = ADVANCED_SIGNATURE;
        if (!StringUtils.isBlank(inSignatureType)){
            signatureType = SignatureType.valueOf(inSignatureType.toUpperCase());
        }
        int securityLevel = 4;
        if (inSecurityLevel == 3){
            securityLevel = 3;
        }else if (inSecurityLevel == 4) {
            NIN = BANKID_NIN;
        }
        else  {
            logger.info("Invalid security level {}, choosing {}", inSecurityLevel, securityLevel);
        }
        logger.info("Using securityType: {}", signatureType);
        logger.info("Using securityLevel: {}", securityLevel);

        return SignRequest.builder()
                .description(description)
                .title(title)
                .existUrls(exitUrls)
                .document(pdfEncoded)
                .signersNin(List.of(NIN))
                .jobType(JobType.direct)
                .signatureType(signatureType)
                .securityLevel(inSecurityLevel)
                .build();

    }

    private RedirectUrlRequest createRedirectUrlRequest(SignerDetail detail) {
        RedirectUrlRequest redirectUrlRequest = RedirectUrlRequest.builder()
            .signerUrl(detail.getSignersRef())
            .build();
        RedirectUrlResponse redirectUrlResponse = signerService.createNewRedirect(redirectUrlRequest);
        return redirectUrlRequest;
    }

}
