package no.edu.esign;

import no.digipost.signature.client.ClientConfiguration;
import no.edu.esign.model.sign.*;
import no.edu.esign.util.FileUtil;
import org.apache.commons.codec.binary.Base64;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.context.SpringBootTest;

import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;

/**
 * To be able to run this test there is need to set following environment:
 * <code>SECRET_CA_FILE_PATH</code>
 * <code>SECRET_CA_FILE_PASSWORD</code>
 */
@SpringBootTest(classes = SignerService.class)
public class SignerServiceIntegrationTest {
    Logger logger = LoggerFactory.getLogger(SignerServiceIntegrationTest.class);
    final String ORGANIZATION_NR_NTNU = "974767880";

    @Value("${ca.file.path}")
    private String filePath;

    @Value("${ca.file.password}")
    private String caPassword;

    @Autowired
    private SignerService signerService;

    @BeforeEach
    public void setup(){
        if ((filePath == null) || (caPassword == null) ||
                (filePath.equals("${SECRET_CA_FILE_PATH}")) || (caPassword.equals("SECRET_CA_FILE_PASSWORD"))){
            throw new RuntimeException("Required to set config via env to be able to run this test");
        }

    }

    @Test
    public void getClientConfiguration() {
        // When
        ClientConfiguration clientConfig = signerService.getClientConfiguration();

        // Then
        assertNotNull(clientConfig);
        assertNotNull(clientConfig.getKeyStoreConfig());
        //noinspection OptionalGetWithoutIsPresent
        assertEquals(ORGANIZATION_NR_NTNU, clientConfig.getGlobalSender().get().getOrganizationNumber());
    }

    @Test
    public void createSign(){
        // Given
        ExitUrls exitUrls = new ExitUrls();
        exitUrls.setCompletion("https://ntnu.no/completed.no");
        exitUrls.setRejection("https://ntnu.no/rejection.no");
        exitUrls.setError("https://ntnu.no/error.no");
        String nin = "65857600716";
        String description = "Signerings tittel";
        String title = "Dokument tittel";
        String fileName = "acrobat_digitalSignatures.pdf";
        String pdfEncoded = Base64.encodeBase64String(FileUtil.getPdfDocAsBytes(fileName));

        // When
        SignRequest signRequest = SignRequest.builder()
                .description(description)
                .title(title)
                .existUrls(exitUrls)
                .document(pdfEncoded)
                .signersNin(List.of(nin))
                .jobType(JobType.direct)
                .signatureType(SignatureType.AUTHENTICATED_SIGNATURE)
                .build();
        SignResponse signResponse = signerService.createSignature(signRequest);

        // Then
        assertNotNull(signResponse.getStatusQueryUrl());
        assertEquals(1, signResponse.getSignersDetails().size());
        SignerDetail detail = signResponse.getSignersDetails().get(0);
        assertNotNull(detail.getNin());
        assertNotNull(detail.getSignersRef());
        assertNotNull(detail.getRedirectUrl());
        String msg = "Created new signing request, redirectUrl: %s, statusUrl: %s";
        logger.info(String.format(msg, detail.getRedirectUrl(), signResponse.getStatusQueryUrl()));
    }

    @Test
    public void getNewRedirectUrl(){
        // Given
        ExitUrls exitUrls = new ExitUrls();
        exitUrls.setCompletion("https://ntnu.no/completed.no");
        exitUrls.setRejection("https://ntnu.no/rejection.no");
        exitUrls.setError("https://ntnu.no/error.no");
        String nin = "65857600716";
        String description = "Signerings tittel";
        String title = "Dokument tittel";
        String fileName = "acrobat_digitalSignatures.pdf";
        String pdfEncoded = Base64.encodeBase64String(FileUtil.getPdfDocAsBytes(fileName));

        // When
        SignRequest signRequest = SignRequest.builder()
            .description(description)
            .title(title)
            .existUrls(exitUrls)
            .document(pdfEncoded)
            .signersNin(List.of(nin))
            .jobType(JobType.direct)
            .signatureType(SignatureType.AUTHENTICATED_SIGNATURE)
            .build();
        SignResponse signResponse = signerService.createSignature(signRequest);
        SignerDetail detail = signResponse.getSignersDetails().get(0);

        RedirectUrlRequest redirectUrlRequest = RedirectUrlRequest.builder()
            .signerUrl(detail.getSignersRef())
            .build();
        RedirectUrlResponse redirectUrlResponse = signerService.createNewRedirect(redirectUrlRequest);

        // Then
        assertNotNull(redirectUrlResponse);
        assertNotNull(redirectUrlResponse.getUpdatedRedirectUrl());
        String msg = "New redirect url is: %s";
        logger.info(String.format(msg, redirectUrlResponse.getUpdatedRedirectUrl()));
    }

}
