package no.edu.esign;

import org.springframework.boot.SpringApplication;
import org.springframework.boot.env.EnvironmentPostProcessor;
import org.springframework.core.env.ConfigurableEnvironment;
import org.springframework.core.env.PropertiesPropertySource;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;
import java.util.TreeSet;

/**
 * Denne klassen må være definert i /META-INF/spring.factories
 * Alle properties som er definert i application.properties må være definert i enten en lokal developerfil, eller i environment-variabler.
 *
 * Properties som starter med secret. er "hemmelige" og skal ikke logges ut. Hvis en property heter secret.my.token i default fila,
 * så heter den bare my.token eller MY_TOKEN i developer fila eller environment.
 *
 * Prinsippet er man kan configurere å ulike vis og de blir lest inn i følgende rekkefølge
 * 1. default properties ligger med i resources/ og definerer all config som skal med. Hvis key starter med secret er den hemmelig og logges ikke.
 * 2. /local/app/conf... den lokale property fil som man kan bruke som utvikler
 * 3. leser environment
 */

public class ConfigLoader implements EnvironmentPostProcessor {
    private static final String DEFAULT_PROPERTIES = "/application.properties";
    private static final String DEVELOPER_PROPERTIES = "/local/app/conf/esign/application.properties";
    public static String CONFIG_LOADED_LOG = "";

    @Override
    public void postProcessEnvironment(ConfigurableEnvironment environment, SpringApplication application) {
        Properties props = loadAll(environment);
        environment.getPropertySources().addLast(new PropertiesPropertySource("ConfigLoader", props));
    }

    @SuppressWarnings("StringConcatenationInsideStringBufferAppend")
    private Properties loadAll(ConfigurableEnvironment environment) {
        try {
            StringBuilder log = new StringBuilder();
            Properties defaultProperties = loadPropertiesFromResource(DEFAULT_PROPERTIES);
            Properties developerProps = loadDeveloperProperties(DEVELOPER_PROPERTIES);
            Set<String> secretPropName = new HashSet<>();
            Set<String> orderedPropNameList = new TreeSet<>();
            Properties finalProps = new Properties();

            for (String key: defaultProperties.stringPropertyNames()){
                String propName = key;
                if (propName.contains("secret") || propName.contains("password")) {
                    secretPropName.add(propName);
                }
                orderedPropNameList.add(propName);
                finalProps.put(propName, defaultProperties.get(key));
            }
            log.append(" **************************************************").append("\n");
            log.append(" *****************  ConfigLoader  *****************").append("\n");
            log.append(" **************************************************").append("\n");
            for (String propName: orderedPropNameList){
                String source = DEFAULT_PROPERTIES;
                String value = finalProps.getProperty(propName);
                String envValue = environment.getProperty(propName);
                String devValue = developerProps.getProperty(propName);

                if (envValue != null && envValue.trim().length()>0){
                    source = "ENV";
                    value = envValue;
                }else if (devValue != null && devValue.trim().length()>0){
                    source = DEVELOPER_PROPERTIES;
                    value = devValue;
                }
                if (secretPropName.contains(propName)){
                    log.append(" " + propName + "=***** [ " + source + " ]").append("\n");
                }else{
                    log.append(" " + propName + "=" + value + " [ " + source + " ]").append("\n");
                }
                if (value == null || value.trim().length() == 0){
                    throw new RuntimeException("Empty property '" + propName + "' must be set in ENV or " + DEVELOPER_PROPERTIES);
                }
                finalProps.setProperty(propName, value);
            }
            log.append(" **************************************************").append("\n");
            CONFIG_LOADED_LOG = log.toString();
            return finalProps;

        } catch (Exception e) {
            throw new RuntimeException("Cant load config: "+ e.getMessage(), e);
        }
    }

    private Properties loadDeveloperProperties(String path){
        Properties props = new Properties();
        try {
            props.load(new FileInputStream(path));
        } catch (IOException e) {
            System.out.println("Skipping developer properies " + path + " " + e.getMessage());
        }
        return props;
    }

    private Properties loadPropertiesFromResource(String path) throws IOException {
        Properties props = new Properties();
        props.load(new InputStreamReader(ConfigLoader.class.getResourceAsStream(path), "UTF-8"));
        return props;
    }
}
