package no.edu.esign;

import no.digipost.signature.client.Certificates;
import no.digipost.signature.client.ClientConfiguration;
import no.digipost.signature.client.ServiceUri;
import no.digipost.signature.client.core.SignatureType;
import no.digipost.signature.client.core.*;
import no.digipost.signature.client.direct.ExitUrls;
import no.digipost.signature.client.direct.*;
import no.digipost.signature.client.security.KeyStoreConfig;
import no.edu.esign.model.sign.*;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


@Service
public class SignerService {
    Logger logger = LoggerFactory.getLogger(SignerService.class);
    private DirectClient client;

    // default scope to be overridden in test
    @Value("${ca.file.path}")
    String caFilePath;
    @Value("${ca.file.password}")
    String caPassword;
    @Value("${edu.organization.nr}")
    String organizationNr;

    @Autowired
    Environment env;

    @PostConstruct
    public void initClient() {
        this.client = new DirectClient(getClientConfiguration());
    }

    public ClientConfiguration getClientConfiguration() {
        KeyStoreConfig keyStoreConfig = createKeyStoreConfig(caFilePath, caPassword);

        Certificates cert = Certificates.PRODUCTION;
        ServiceUri serviceUri = ServiceUri.PRODUCTION;
        if (Arrays.asList(env.getActiveProfiles()).contains("dev")) {
            cert = Certificates.TEST;
            serviceUri = ServiceUri.DIFI_TEST;
        }
        else if (Arrays.asList(env.getActiveProfiles()).contains("qa")) {
            cert = Certificates.TEST;
            serviceUri = ServiceUri.DIFI_QA;
        }
        logger.info("Loaded cert for ************ {} ENV ************", cert.toString());

        return ClientConfiguration.builder(keyStoreConfig)
                .trustStore(cert)
                .serviceUri(serviceUri)
                .globalSender(new Sender(organizationNr))
                .build();
    }

    private KeyStoreConfig createKeyStoreConfig(String caFilePath, String caFilePassword) {
        KeyStoreConfig keyStoreConfig;
        try (InputStream certificateStream = Files.newInputStream(Paths.get(caFilePath))) {
            keyStoreConfig = KeyStoreConfig.fromOrganizationCertificate(
                    certificateStream, caFilePassword
            );
        } catch (IOException e) {
            throw new IllegalArgumentException(String.format("Could not create keystore from file: %s", caFilePath));
        }
        return keyStoreConfig;
    }

    public SignResponse createSignature(SignRequest request) {
        byte[] documentBytes = Base64.decodeBase64(request.getDocument().getBytes());
        DirectDocument document = DirectDocument.builder(request.getTitle(), documentBytes).build();

        no.edu.esign.model.sign.ExitUrls existUrls = request.getExistUrls();
        ExitUrls exitUrls = ExitUrls.of(
                URI.create(existUrls.getCompletion()),
                URI.create(existUrls.getRejection()),
                URI.create(existUrls.getError())
        );
        List<DirectSigner> directSigners = new ArrayList<>();
        SignatureType signatureType = getSignatureTypeOrThrow(request);


        for (String nin : request.getSignersNin()) {
            DirectSigner directSigner = DirectSigner.withPersonalIdentificationNumber(nin)
                    .withSignatureType(signatureType)
                    .build();

            directSigners.add(directSigner);
        }

        AuthenticationLevel authLevel = request.getSecurityLevel() == 3 ? AuthenticationLevel.THREE : AuthenticationLevel.FOUR;
        DirectJob directJob = DirectJob.builder(request.getDescription(),
                        List.of(document),
                        directSigners,
                        exitUrls)
                .requireAuthentication(authLevel)
                .build();
        DirectJobResponse directJobResponse = client.create(directJob);
        logger.debug(String.format("Created new direct sign request, signatureJobId: %d", directJobResponse.getSignatureJobId()));
        return SignResponse.builder()
                .signatureJobId(String.valueOf(directJobResponse.getSignatureJobId()))
                .statusQueryUrl(directJobResponse.getStatusUrl().toString())
                .signersDetails(getSignersDetails(directJobResponse.getSigners()))
                .build();

    }

    public RedirectUrlResponse createNewRedirect(RedirectUrlRequest redirectUrlRequest) {
        DirectSignerResponse signerWithUpdatedRedirectUrl = client
            .requestNewRedirectUrl(WithSignerUrl.of(URI.create(redirectUrlRequest.getSignerUrl())));
        String updatedRedirectUrl= signerWithUpdatedRedirectUrl.getRedirectUrl().toString();
        return RedirectUrlResponse.builder()
            .updatedRedirectUrl(updatedRedirectUrl)
            .build();
    }

    public StatusResponse getStatus(StatusRequest request) {
        DirectJobStatusResponse directJobStatusResponse = client
                .getStatus(StatusReference.ofUrl(URI.create(request.getStatusQueryUrl()))
                        .withStatusQueryToken(request.getStatusQueryToken())
                );
        String padesUrl = directJobStatusResponse.isPAdESAvailable() ? directJobStatusResponse.getpAdESUrl().getpAdESUrl().toString() : null;
        return StatusResponse.builder()
                .status(directJobStatusResponse.getStatus())
                .confirmationUrl(String.valueOf(directJobStatusResponse.getConfirmationReference().getConfirmationUrl()))
                .padesUrl(padesUrl)
                .build();
    }

    private List<SignerDetail> getSignersDetails(List<DirectSignerResponse> signerResp) {
        List<SignerDetail> ninUrls = new ArrayList<>();
        for (DirectSignerResponse signer : signerResp) {
            ninUrls.add(new SignerDetail(signer.getPersonalIdentificationNumber(),
                    signer.getSignerUrl().toString(),
                    signer.getRedirectUrl().toString()));
        }
        return ninUrls;
    }

    public DocumentResponse getDocuments(DocumentRequest request) {
        byte[] bytes = getDocumentBytes(request);
        if (bytes == null) {
            return null;
        }
        String document = new String(Base64.encodeBase64(bytes));
        return DocumentResponse.builder().document(document).build();
    }

    private byte[] getDocumentBytes(DocumentRequest request) {
        try {
            ResponseInputStream input = null;
            if (!StringUtils.isBlank(request.getPadesUrl())) {
                input = client.getPAdES(PAdESReference.of(URI.create(request.getPadesUrl())));
            } else {
                logger.info("PadesUrl is empty, trying to get document from xadesUrl");
                input = client.getXAdES(XAdESReference.of(URI.create(request.getXadesUrl())));
            }
            if (input == null) {
                logger.warn("Cant get document, invalid padesUrl or xadesUrl, request: {}", request);
                return null;
            }
            return input.readAllBytes();
        } catch (IOException e) {
            throw new RuntimeException("Could not read document", e);
        } catch (Exception se) {
            if (se.getCause().toString().startsWith("java.net.UnknownHostException")) {
                throw new IllegalArgumentException("Invalid pades Url: " + request.getPadesUrl());
            }
            throw new RuntimeException("Could not get document", se);
        }
    }

    private SignatureType getSignatureTypeOrThrow(SignRequest request) {
        SignatureType signatureType = null;
        try{
            signatureType = SignatureType.valueOf(request.getSignatureType().name());
        }catch (Exception se){
            throw new IllegalArgumentException("Invalid signatureType, should be one of: AUTHENTICATED_SIGNATURE or ADVANCED_SIGNATURE");
        }
        return signatureType;
    }
}
