package no.edu.esign.controller;


import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import io.swagger.v3.oas.annotations.tags.Tag;
import no.edu.esign.SignerService;
import no.edu.esign.model.sign.*;
import no.edu.esign.util.AssertionsUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.ws.rs.Consumes;
import javax.ws.rs.Produces;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import static no.edu.esign.util.AssertionsUtil.assertSignRequest;

@Tag(name = "eSigning API", description = "Use this api to create new signing job, pull for status and download signed document")
@RestController
@RequestMapping("/api")
@Produces({"application/json"})
@Consumes({"application/json"})
public class     SignController {
    Logger logger = LoggerFactory.getLogger(SignController.class);

    @Autowired
    private SignerService signerService;

    @GetMapping(path = "/healthz", produces = MediaType.TEXT_PLAIN_VALUE)
    public String healtz() {
        return "healthz check passed";
    }


    @Operation(summary = "Creates new signing job")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "New signing job is created",
                    content = {@Content(mediaType = "application/json",
                            schema = @Schema(implementation = SignResponse.class))}),
            @ApiResponse(responseCode = "400", description = "Bad request",
                    content = @Content),
    })
    @PostMapping(path = "/create", produces = MediaType.APPLICATION_JSON_VALUE,
            consumes = MediaType.APPLICATION_JSON_VALUE)
    public @ResponseBody
    ResponseEntity<Object> createSign(@Valid @RequestBody SignRequest request) {
        try {
            assertSignRequest(request);

            List<String> nins = request.getSignersNin()
                    .stream()
                    .map(x -> x.substring(0, 6))
                    .collect(Collectors.toList());
            logger.info("Creating sign request, signers: " + nins);

            return ResponseEntity.ok(signerService.createSignature(request));
        } catch (IllegalArgumentException ie) {
            logger.error("Could not create sign request", ie);
            return badRequest(ie);
        }
    }

    @Operation(summary = "Get status of created signing job")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Status of signing job",
                    content = {@Content(mediaType = "application/json",
                            schema = @Schema(implementation = StatusResponse.class))}),
            @ApiResponse(responseCode = "400", description = "Bad request",
                    content = @Content),
    })
    @PostMapping(path = "/status", produces = MediaType.APPLICATION_JSON_VALUE, consumes = MediaType.APPLICATION_JSON_VALUE)
    public @ResponseBody
    StatusResponse checkStatus(@Valid @RequestBody StatusRequest request) {
        return signerService.getStatus(request);
    }

    @PostMapping(path = "/redirectUrl", produces = MediaType.APPLICATION_JSON_VALUE, consumes = MediaType.APPLICATION_JSON_VALUE)
    public @ResponseBody
    RedirectUrlResponse getRedirectUrl(@Valid @RequestBody RedirectUrlRequest request) {
        return signerService.createNewRedirect(request);
    }

    @Operation(summary = "Get signed document")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Base 64 encoded pdf files",
                    content = {@Content(mediaType = "application/json",
                            schema = @Schema(implementation = DocumentResponse.class))}),
            @ApiResponse(responseCode = "400", description = "Bad request",
                    content = @Content),
    })
    @PostMapping(path = "/document", produces = MediaType.APPLICATION_JSON_VALUE,
            consumes = MediaType.APPLICATION_JSON_VALUE)
    public @ResponseBody
    ResponseEntity<Object> document(@Valid @RequestBody DocumentRequest request) {
        logger.info("Getting document");
        try {
            AssertionsUtil.assertDocumentRequest(request);

            DocumentResponse docResp = signerService.getDocuments(request);
            if (docResp == null){
                return notFoundDocument();
            }

            return ResponseEntity.ok(docResp);

        } catch (IllegalArgumentException ie) {
            return badRequest(ie);
        }
    }

    private ResponseEntity<Object> badRequest(IllegalArgumentException ie) {
        return ResponseEntity.badRequest()
                .body(ErrorMsg.builder()
                        .message(ie.getMessage())
                        .status(HttpStatus.BAD_REQUEST.value())
                        .error(HttpStatus.BAD_REQUEST.getReasonPhrase())
                        .timestamp(new Date())
                        .build()
                );
    }
    private ResponseEntity<Object> notFoundDocument() {
        return ResponseEntity.badRequest()
                .body(ErrorMsg.builder()
                        .message(String.format("Can't find any %s", "document"))
                        .status(HttpStatus.NOT_FOUND.value())
                        .error(HttpStatus.NOT_FOUND.getReasonPhrase())
                        .timestamp(new Date())
                        .build()
                );
    }
}
