package no.edu.esign.controller;


import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import no.edu.esign.SignerService;
import no.edu.esign.model.identity.AccessTokenResponse;
import no.edu.esign.model.sign.*;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockHttpServletResponse;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.ResultHandler;

import java.util.List;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.doReturn;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest
@AutoConfigureMockMvc
public class SignControllerTest {

    @Value("${jwt.client.gravitee.secret}")
    String clientSecret;

    @Value("${jwt.client.gravitee.scope}")
    String scope;

    @Value("${jwt.expiry.millis}")
    int expiryMillis;

    @Autowired
    private MockMvc mvc;

    @MockBean
    private SignerService signerService;

    @Autowired
    private ObjectMapper objectMapper;

    final AccessTokenResponse[] accessTokenResp = new AccessTokenResponse[1];

    @Test
    public void docs() throws Exception {
        mvc.perform(get("/docs"))
                .andExpect(status().is3xxRedirection())
                .andDo(new ResultHandler() {
                    @Override
                    public void handle(MvcResult result) throws Exception {
                        assertNotNull(result.getResponse());
                        assertEquals("/swagger-ui/index.html", result.getResponse().getRedirectedUrl());
                    }
                });

    }

    @Test
    public void accessToken() throws Exception {
        AccessTokenResponse accResp = null;
        mvc.perform(post("/api/identity/token")
                        .contentType(MediaType.APPLICATION_JSON_VALUE)
                        .content(body_access_token()))
                .andExpect(status().isOk())
                .andDo(new ResultHandler() {
                    @Override
                    public void handle(MvcResult result) throws Exception {
                        MockHttpServletResponse response = result.getResponse();
                        String contentAsString = response.getContentAsString();
                        assertAccessToken(contentAsString);
                    }
                });
    }

    @Test
    public void accessToken_unauthorized() throws Exception {
        mvc.perform(post("/api/identity/token")
                        .contentType(MediaType.APPLICATION_JSON_VALUE)
                        .content("""
                                { "scope":"api.sign",
                                 "client": "gravitee",
                                 "client_secret": "wrong_secret"
                                 }
                                 """))
                .andExpect(status().isUnauthorized());
    }

    @Test
    public void health_check() throws Exception {
        AccessTokenResponse tokenResp = getAccessToken();
        mvc.perform(get("/api/healthz"))
                .andExpect(status().isOk());

    }

    @Test
    public void createSignature_ok_and_un_authorized() throws Exception {
        AccessTokenResponse tokenResp = getAccessToken();
        String statusQueryUrl = "https://api.difitest.signering.posten.no/api/974767880/direct/signature-jobs/71367/status";
        String nin = "65857600716";
        String signersRef = "https://difitest.signering.posten.no/signere/-/uoWKia-dmrqO5DwWrdfKIzBzOwQn7BEfwn3O8-lqY4U";
        String signersRedirectUrl = "https://api.difitest.signering.posten.no/api/974767880/direct/signature-jobs/71367/signers/77222";

        final SignResponse response = SignResponse.builder()
                .signatureJobId("1")
                .statusQueryUrl(statusQueryUrl)
                .signersDetails(List.of(new SignerDetail(nin, signersRef, signersRedirectUrl)))
                .build();

        doReturn(response).when(signerService).createSignature(any(SignRequest.class));

        mvc.perform(post("/api/create")
                        .contentType(MediaType.APPLICATION_JSON)
                        .header("Authorization", "Bearer " + tokenResp.getAccessToken())
                        .content(bodyCreateSignature()))
                .andExpect(status().isOk())
                .andExpect(content().json(objectMapper.writeValueAsString(response)));

        // test unAuthorized
        mvc.perform(post("/api/create")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(bodyCreateSignature()))
                .andExpect(status().isUnauthorized());

    }
    @Test
    public void get_redirectUrl_ok_and_un_authorized() throws Exception {
        AccessTokenResponse tokenResp = getAccessToken();
        String signersRef = "https://difitest.signering.posten.no/signere/-/uoWKia-dmrqO5DwWrdfKIzBzOwQn7BEfwn3O8-lqY4U";

        RedirectUrlResponse response = RedirectUrlResponse.builder()
            .updatedRedirectUrl(signersRef)
            .build();
        doReturn(response).when(signerService).createNewRedirect(any(RedirectUrlRequest.class));

        mvc.perform(post("/api/redirectUrl")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer " + tokenResp.getAccessToken())
                .content(bodySignerUrlReq()))
            .andExpect(status().isOk());

        mvc.perform(post("/api/redirectUrl")
                .contentType(MediaType.APPLICATION_JSON)
                .content(bodySignerUrlReq()))
            .andExpect(status().isUnauthorized());
    }

    @Test
    public void createSignature_bad_request() throws Exception {
        AccessTokenResponse tokenResp = getAccessToken();
        String statusQueryUrl = "https://api.difitest.signering.posten.no/api/974767880/direct/signature-jobs/71367/status";
        String signersRef = "https://difitest.signering.posten.no/signere/-/uoWKia-dmrqO5DwWrdfKIzBzOwQn7BEfwn3O8-lqY4U";
        String signersRedirectUrl = "https://api.difitest.signering.posten.no/api/974767880/direct/signature-jobs/71367/signers/77222";
        String nin = null;
        final SignResponse response = SignResponse.builder()
                .signatureJobId("1")
                .statusQueryUrl(statusQueryUrl)
                .signersDetails(List.of(new SignerDetail(nin, signersRef, signersRedirectUrl)))
                .build();

        doReturn(response).when(signerService).createSignature(any(SignRequest.class));

        mvc.perform(post("/api/create")
                        .header("Authorization", "Bearer " + tokenResp.getAccessToken())
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(body_missing_min()))
                .andExpect(status().isBadRequest());

    }

    @Test
    public void status_ok() throws Exception {
        AccessTokenResponse accessTokenResp = getAccessToken();

        StatusResponse docResp = StatusResponse.builder()
                .confirmationUrl("http://example.com/confirmationUrl")
                .padesUrl("https://example.com/padesUrl.com")
                .build();

        doReturn(docResp).when(signerService).getStatus(any(StatusRequest.class));

        mvc.perform(post("/api/status")
                        .header("Authorization", "Bearer " + accessTokenResp.getAccessToken())
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(bodyStatusReq()))
                .andExpect(status().isOk())
                .andExpect(content().json(objectMapper.writeValueAsString(docResp)));

    }

    @Test
    public void document_ok() throws Exception {
        AccessTokenResponse accessTokenResp = getAccessToken();

        DocumentResponse docResp = DocumentResponse.builder()
                .document("document")
                .build();

        doReturn(docResp).when(signerService).getDocuments(any(DocumentRequest.class));

        mvc.perform(post("/api/document")
                        .header("Authorization", "Bearer " + accessTokenResp.getAccessToken())
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(bodyDocumentReq()))
                .andExpect(status().isOk())
                .andExpect(content().json(objectMapper.writeValueAsString(docResp)));

    }

    @Test
    public void document_missing_required_pades_url() throws Exception {
        AccessTokenResponse tokenResp = getAccessToken();
        DocumentResponse docResp = DocumentResponse.builder()
                .document("document")
                .build();

        doReturn(docResp).when(signerService).getDocuments(any(DocumentRequest.class));

        mvc.perform(post("/api/document")
                        .header("Authorization", "Bearer " + tokenResp.getAccessToken())
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(""" 
                                { "PadesURL": "" "XadesURL": "" } """))
                .andExpect(status().isBadRequest());

    }

    private AccessTokenResponse getAccessToken() throws Exception {
        mvc.perform(post("/api/identity/token")
                        .contentType(MediaType.APPLICATION_JSON_VALUE)
                        .content(body_access_token()))
                .andExpect(status().isOk())
                .andDo(new ResultHandler() {
                    @Override
                    public void handle(MvcResult result) throws Exception {
                        MockHttpServletResponse response = result.getResponse();
                        String contentAsString = response.getContentAsString();
                        accessTokenResp[0] = objectMapper.readValue(contentAsString, AccessTokenResponse.class);
                    }
                });
        return accessTokenResp[0];
    }

    private void assertAccessToken(String content) throws JsonProcessingException {
        assertTrue(content.contains("access_token"));
        assertTrue(content.contains("expires_in"));
        assertTrue(content.contains("token_type"));
        AccessTokenResponse accResp = objectMapper.readValue(content, AccessTokenResponse.class);
        assertNotNull(accResp.getAccessToken());
        assertEquals("Bearer", accResp.getTokenType());
        assertEquals(expiryMillis, accResp.getExpiresInMillis());
    }

    private String bodyStatusReq() {
        return """ 
                {
                    "StatusQueryURL": "https://api.difitest.signering.posten.no/api/974767880/direct/signature-jobs/71960/status",
                    "StatusQueryToken": "t0JBT-3O_Cerg_JZPHIj1HnJoN9ZL8P4y4E3YHEaUAM"
                }
                """;
    }

    private String bodySignerUrlReq() {
        return """
               {"SignerUrl": "https://api.difitest.signering.posten.no/api/974767880/direct/signature-jobs/74776/signers/80929"}
               """;
    }

    private String bodyDocumentReq() {
        return """ 
                { "PadesURL": "https://padesUrl.com" ,
                "XadesURL": "https://XadesUrl.com"}
                """;
    }

    public String bodyCreateSignature() {
        return """
                {   
                    "SignatureType": "AUTHENTICATED_SIGNATURE",
                    "JobType": "direct",
                    "SecurityLevel": 4,
                    "Title": "Dokument tittel",
                    "Description": "Signerings tittel",
                    "SignersNin": [
                        "65857600716"
                    ],
                    "ExistUrls": {
                        "Completion": "https://ntnu.no/completed",
                        "Rejection": "https://ntnu.no/rejection",
                        "Error": "https://ntnu.no/error"
                    },
                    "Document": "JVBERi0xLjYNJeLjz9MNCjUxNSAwIG9iag08PC9MaW5lYXJpemVkIDEvTCAzMzg0MDAvTyA1MTcvRSAyOTY0Ni9OIDE0L1QgMzM3NTMyL0ggWyA1OTcgNDE2XT4+D"
                }
                """;
    }

    public String body_missing_min() {
        return """
                {
                    "JobType": "direct",
                    "SecurityLevel": 4,
                    "Title": "Dokument tittel",
                    "Description": "Signerings tittel",
                    
                    "ExistUrls": {
                        "completion": "https://ntnu.no/completed",
                        "rejection": "https://ntnu.no/rejection",
                        "error": "https://ntnu.no/error"
                    },
                    "Document": "JVBERi0xLjYNJeLjz9MNCjUxNSAwIG9iag08PC9MaW5lYXJpemVkIDEvTCAzMzg0MDAvTyA1MTcvRSAyOTY0Ni9OIDE0L1QgMzM3NTMyL0ggWyA1OTcgNDE2XT4+D"
                }
                """;
    }

    public String body_access_token() {
        return """
                {
                    "scope":"api.sign",
                    "client": "gravitee",
                    "client_secret": "foo"
                }
                """;
    }
}
